import 'package:flutter/material.dart';

enum AnimationType { normal, fadeIn, slide }

class AnimateNavigationTrasitionDo {
  final BuildContext context;
  final Widget childPage;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  AnimateNavigationTrasitionDo(
      {required this.context,
      required this.childPage,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 300),
      this.replacement = false}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        _fadeInTransition();
        break;
      case AnimationType.slide:
        _slideTransition();
        break;
    }
  }

  void _pushPage(Route route) => Navigator.push(context, route);

  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.childPage);

    (this.replacement) ? _pushReplacementPage(route) : _pushPage(route);
  }

  void _fadeInTransition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                CurvedAnimation(parent: animation, curve: Curves.easeOut)),
          );
        });

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _slideTransition() {
    final transition = PageRouteBuilder(
      pageBuilder: (_, __, ___) => this.childPage,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) => SlideTransition(
        position: Tween<Offset>(
          begin: const Offset(-1, 0),
          end: Offset.zero,
        ).animate(animation),
        child: child,
      ),
    );

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }
}
